@@ -2,12 +2,12 @@
 #'
 #' @param ... A set of name-value pairs. The names of "key" and "index" should
 #' be avoided as they are used as the arguments.
-#' @param key Variable(s) that define unique time indices, used in conjunction
-#' with the helper [id()]. If a univariate time series (without an explicit key),
-#' simply call `id()`.
+#' @param key Unquoted variable(s) that uniquely determine time indices. `NULL` for
+#' empty key, and works with tidy selector (e.g. `starts_with()`).
 #' @param index A bare (or unquoted) variable to specify the time index variable.
 #' @param regular Regular time interval (`TRUE`) or irregular (`FALSE`). The
 #' interval is determined by the greatest common divisor of index column, if `TRUE`.
+#' @param .drop If `TRUE`, empty key groups are dropped.
 #'
 #' @inheritSection tsibble-package Index
 #'
@@ -21,29 +21,30 @@
 #' @seealso [build_tsibble]
 #'
 #' @examples
-#' # create a tsibble w/o a key ----
+#' # create a tsibble w/o a key
 #' tsibble(
 #'   date = as.Date("2017-01-01") + 0:9,
 #'   value = rnorm(10)
 #' )
 #'
-#' # create a tsibble with one key ----
+#' # create a tsibble with one key
 #' tsibble(
 #'   qtr = rep(yearquarter("201001") + 0:9, 3),
 #'   group = rep(c("x", "y", "z"), each = 10),
 #'   value = rnorm(30),
-#'   key = id(group)
+#'   key = group
 #' )
-#'
 #' @export
-tsibble <- function(..., key = id(), index, regular = TRUE) {
+tsibble <- function(..., key = NULL, index, regular = TRUE, .drop = TRUE) {
+  stopifnot(is_logical(regular, n = 1))
   dots <- list2(...)
   if (has_length(dots, 1) && is.data.frame(dots[[1]])) {
     abort("Must not be a data frame, do you want `as_tsibble()`?")
   }
   tbl <- tibble(!!! dots)
   index <- enquo(index)
-  build_tsibble(tbl, key = !! enquo(key), index = !! index, regular = regular)
+  build_tsibble(tbl, key = !! enquo(key), index = !! index, interval = regular,
+    .drop = .drop)
 }
 
 #' Coerce to a tsibble object
@@ -60,39 +61,41 @@ tsibble <- function(..., key = id(), index, regular = TRUE) {
 #' @seealso [tsibble]
 #'
 #' @examples
-#' # coerce tibble to tsibble w/o a key ----
+#' # coerce tibble to tsibble w/o a key
 #' tbl1 <- tibble(
 #'   date = as.Date("2017-01-01") + 0:9,
 #'   value = rnorm(10)
 #' )
 #' as_tsibble(tbl1)
-#' # specify the index var
+#' # supply the index to suppress the message
 #' as_tsibble(tbl1, index = date)
 #'
-#' # coerce tibble to tsibble with one key ----
+#' # coerce tibble to tsibble with one key
 #' # "date" is automatically considered as the index var, and "group" is the key
 #' tbl2 <- tibble(
 #'   mth = rep(yearmonth("2017-01") + 0:9, 3),
 #'   group = rep(c("x", "y", "z"), each = 10),
 #'   value = rnorm(30)
 #' )
-#' as_tsibble(tbl2, key = id(group))
-#' as_tsibble(tbl2, key = id(group), index = mth)
-#'
+#' as_tsibble(tbl2, key = group)
+#' as_tsibble(tbl2, key = group, index = mth)
 #' @export
-as_tsibble <- function(x, ...) {
+as_tsibble <- function(x, key = NULL, index, regular = TRUE, 
+  validate = TRUE, .drop = TRUE, ...
+) {
+  stopifnot(is_logical(regular, n = 1))
   UseMethod("as_tsibble")
 }
 
-#' @rdname as-tsibble
+#' @keywords internal
 #' @export
-as_tsibble.tbl_df <- function(
-  x, key = id(), index, regular = TRUE, validate = TRUE, ...
+as_tsibble.tbl_df <- function(x, key = NULL, index, regular = TRUE, 
+  validate = TRUE, .drop = TRUE, ...
 ) {
   index <- enquo(index)
   build_tsibble(
-    x, key = !! enquo(key), index = !! index, regular = regular,
-    validate = validate
+    x, key = !! enquo(key), index = !! index, interval = regular,
+    validate = validate, .drop = .drop
   )
 }
 
@@ -103,7 +106,7 @@ as_tsibble.data.frame <- as_tsibble.tbl_df
 #' @keywords internal
 #' @export
 as_tsibble.tbl_ts <- function(x, ...) {
-  x
+  update_tsibble(x, ...)
 }
 
 #' @keywords internal
@@ -116,13 +119,13 @@ as_tsibble.list <- as_tsibble.tbl_df
 
 #' @keywords internal
 #' @export
-as_tsibble.grouped_df <- function(
-  x, key = id(), index, regular = TRUE, validate = TRUE, ...
+as_tsibble.grouped_df <- function(x, key = NULL, index, regular = TRUE, 
+  validate = TRUE, .drop = key_drop_default(x), ...
 ) {
   index <- enquo(index)
   build_tsibble(
-    x, key = !! enquo(key), index = !! index,
-    regular = regular, validate = validate
+    x, key = !! enquo(key), index = !! index, interval = regular, 
+    validate = validate, .drop = .drop
   )
 }
 
@@ -147,180 +150,47 @@ as_tsibble.NULL <- function(x, ...) {
 #' @param x A tsibble.
 #' @inheritParams as_tsibble
 #' @details
-#' Default `NULL` inherits attributes from `x`.
+#' Unspecified arguments will inherit the attributes from `x`.
 #' @export
 #' @examples
+#' library(dplyr)
 #' pedestrian %>% 
 #'   group_by_key() %>% 
 #'   mutate(Hour_Since = Date_Time - min(Date_Time)) %>% 
 #'   update_tsibble(index = Hour_Since)
-update_tsibble <- function(x, key = NULL, index = NULL, regular = NULL, 
-  validate = TRUE) {
+update_tsibble <- function(x, key, index, regular = is_regular(x), 
+  validate = TRUE, .drop = key_drop_default(x)) {
+  not_tsibble(x)
   key <- enquo(key)
-  if (quo_is_null(key)) {
-    key <- key_data(x)
+  if (quo_is_missing(key)) {
+    key <- key_vars(x)
   }
   idx <- enquo(index)
-  if (quo_is_null(idx)) {
-    idx <- x %@% "index"
+  if (quo_is_missing(idx)) {
+    idx <- index_var(x)
   }
-  if (is_null(regular)) {
-    regular <- is_regular(x)
+  stopifnot(is_logical(regular, n = 1))
+  if (is_true(regular)) {
+    int <- interval(x)
+    int <- if (unknown_interval(int)) int else TRUE
+  } else {
+    int <- FALSE
   }
 
-  is_idx_idx2 <- identical(x %@% "index", index2(x))
+  is_idx_idx2 <- identical(index_var(x), index2_var(x))
   if (is_idx_idx2) {
     build_tsibble(
-      as_grouped_df(x), key = !! key, index = !! idx,
-      regular = regular, validate = validate
+      as_tibble(x), key = !! key, index = !! idx,
+      interval = int, validate = validate, .drop = .drop
     )
   } else {
     build_tsibble(
-      as_grouped_df(x), key = !! key, index = !! idx, index2 = !! index2(x),
-      regular = regular, validate = validate
+      as_tibble(x), key = !! key, index = !! idx, index2 = !! index2(x),
+      interval = int, validate = validate, .drop = .drop
     )
   }
 }
 
-#' Return measured variables
-#'
-#' @param x A `tbl_ts`.
-#'
-#' @rdname measured-vars
-#' @examples
-#' measures(pedestrian)
-#' measures(tourism)
-#'
-#' measured_vars(pedestrian)
-#' measured_vars(tourism)
-#' @export
-measures <- function(x) {
-  UseMethod("measures")
-}
-
-#' @export
-measures.tbl_ts <- function(x) {
-  syms(measured_vars(x))
-}
-
-#' @rdname measured-vars
-#' @export
-measured_vars <- function(x) {
-  UseMethod("measured_vars")
-}
-
-#' @export
-measured_vars.tbl_ts <- function(x) {
-  all_vars <- dplyr::tbl_vars(x)
-  key_vars <- key_vars(x)
-  idx_var <- index_var(x)
-  setdiff(all_vars, c(key_vars, idx_var))
-}
-
-#' Return index variable from a tsibble
-#'
-#' @param x A tsibble object.
-#' @rdname index-rd
-#' @examples
-#' index(pedestrian)
-#' index_var(pedestrian)
-#' @export
-index <- function(x) {
-  not_tsibble(x)
-  x %@% "index"
-}
-
-#' @rdname index-rd
-#' @export
-index_var <- function(x) {
-  as_string(index(x))
-}
-
-#' @rdname index-rd
-#' @export
-index2 <- function(x) {
-  not_tsibble(x)
-  x %@% "index2"
-}
-
-#' @rdname index-rd
-#' @export
-index2_var <- function(x) {
-  as_string(index2(x))
-}
-
-#' Meta-information of a tsibble
-#'
-#' * `interval()` returns an interval of a tsibble.
-#' * `is_regular` checks if a tsibble is spaced at regular time or not.
-#' * `is_ordered` checks if a tsibble is ordered by key and index.
-#'
-#' @param x A tsibble object.
-#' @rdname regular
-#' @examples
-#' interval(pedestrian)
-#' @export
-interval <- function(x) {
-  not_tsibble(x)
-  x %@% "interval"
-}
-
-#' @rdname regular
-#' @examples
-#' is_regular(pedestrian)
-#' is_ordered(pedestrian)
-#' @export
-is_regular <- function(x) {
-  not_tsibble(x)
-  x %@% "regular"
-}
-
-#' @rdname regular
-#' @export
-is_ordered <- function(x) {
-  not_tsibble(x)
-  x %@% "ordered"
-}
-
-#' If the object is a tsibble
-#'
-#' @param x An object.
-#'
-#' @return TRUE if the object inherits from the tbl_ts class.
-#' @rdname is-tsibble
-#' @aliases is.tsibble
-#' @examples
-#' # A tibble is not a tsibble ----
-#' tbl <- tibble(
-#'   date = seq(as.Date("2017-10-01"), as.Date("2017-10-31"), by = 1),
-#'   value = rnorm(31)
-#' )
-#' is_tsibble(tbl)
-#'
-#' # A tsibble ----
-#' tsbl <- as_tsibble(tbl, index = date)
-#' is_tsibble(tsbl)
-#' @export
-is_tsibble <- function(x) {
-  inherits(x, "tbl_ts")
-}
-
-#' @rdname is-tsibble
-#' @usage NULL
-#' @export
-is.tsibble <- is_tsibble
-
-#' @rdname is-tsibble
-#' @export
-is_grouped_ts <- function(x) {
-  inherits(x, "grouped_ts")
-}
-
-#' @rdname is-tsibble
-#' @usage NULL
-#' @export
-is.grouped_ts <- is_grouped_ts
-
 ## tsibble is a special class of tibble that handles temporal data. It
 ## requires a sequence of time index to be unique across every identifier.
 
@@ -332,146 +202,130 @@ is.grouped_ts <- is_grouped_ts
 #'
 #' @param x A `data.frame`, `tbl_df`, `tbl_ts`, or other tabular objects.
 #' @inheritParams as_tsibble
+#' @param key_data A data frame containing key variables and `.rows`. When a data
+#' frame is supplied, the argument `key` will be ignored.
 #' @param index2 A candidate of `index` to update the index to a new one when
 #' [index_by]. By default, it's identical to `index`.
 #' @param ordered The default of `NULL` arranges the key variable(s) first and
 #' then index from past to future. `TRUE` suggests to skip the ordering as `x` in
-#' the correct order. `FALSE` also skips the ordering but gives a warning instead.
-#' @param interval `NULL` computes the interval. Use the specified interval via
-#' [new_interval()] as is, if an class of `interval` is supplied.
+#' the correct order. `FALSE` checks the ordering and may give a warning.
+#' @param interval `TRUE` automatically calculates the interval, and `FALSE` for
+#' irregular interval. Use the specified interval via [new_interval()] as is.
 #'
 #' @export
 #' @examples
 #' # Prepare `pedestrian` to use a new index `Date` ----
 #' pedestrian %>%
 #'   build_tsibble(
-#'     key = key(.), index = !! index(.), index2 = Date, interval = interval(.)
+#'     key = !! key_vars(.), index = !! index(.), index2 = Date,
+#'     interval = interval(.)
 #'   )
 build_tsibble <- function(
-  x, key, index, index2, ordered = NULL, regular = TRUE, interval = NULL,
-  validate = TRUE
+  x, key, key_data = NULL, index, index2, ordered = NULL, interval = TRUE,
+  validate = TRUE, .drop = key_drop_default(x)
 ) {
-  # if key is quosures
-  key_sym <- use_id(x, !! enquo(key))
-  is_key_data <- is.data.frame(key_sym)
+  is_key_data <- !is_null(key_data)
   if (is_key_data) {
-    key_data <- key_sym
-    key_sym <- head(names(key_data), -1L)
-  }
-
-  is_grped <- dplyr::is_grouped_df(x)
-  if (is_grped) {
-    tbl <- x
-  } else {
-    tbl <- as_tibble(x)
+    assert_key_data(key_data)
+    key <- head(names(key_data), -1L)
   }
+  key_sym <- use_id(x, !! enquo(key))
+  key_vars <- syms(unname(key_sym))
 
+  tbl <- as_tibble(x)
   # extract or pass the index var
   qindex <- enquo(index)
-  index <- validate_index(tbl, qindex)
+  index <- validate_index(tbl, !! qindex)
   # if index2 not specified
   index2 <- enquo(index2)
-  if (quo_is_missing(index2)) {
+  if (quo_is_missing(index2) || identical(qindex, index2)) {
     index2 <- index
-  } else if (identical(qindex, index2)) {
-    index2 <- get_expr(index2)
   } else {
-    index2 <- validate_index(tbl, index2)
+    index2 <- validate_index(tbl, !! index2)
   }
-  # (1) validate and process key vars (from expr to a list of syms)
-  key_vars <- validate_key(tbl, key_sym)
-  # (2) index cannot be part of the keys
-  idx_chr <- c(as_string(index), as_string(index2))
+  # index cannot be part of the keys
+  idx_chr <- c(index, index2)
   is_index_in_keys <- intersect(idx_chr, key_vars)
   if (is_false(is_empty(is_index_in_keys))) {
     abort(sprintf("Column `%s` can't be both index and key.", idx_chr[[1]]))
   }
-  # arrange in time order (by key and index)
-  if (is_null(ordered)) { # first time to create a tsibble
-    tbl <- arrange(tbl, !!! key_vars, !! index)
+  # arrange index from past to future for each key value
+  if (NROW(tbl) == 0 || is_null(ordered)) { # first time to create a tsibble
+    tbl <- arrange(tbl, !!! key_vars, !! sym(index))
     ordered <- TRUE
-  } else if (is_false(ordered)) { # false returns a warning
-    msg_header <- "Unexpected temporal ordering. Please sort by %s."
-    idx_txt <- expr_text(index)
-    if (is_empty(key)) {
-      warn(sprintf(msg_header, idx_txt))
-    } else {
-      warn(sprintf(msg_header, paste_comma(c(key_vars, idx_txt))))
-    }
-  } # true do nothing
-  # validate tbl_ts
-  if (!is_key_data) {
-    key_vars <- unname(key_vars)
-    key_data <- group_data(group_by(tbl, !!! key_vars))
   }
+  if (!is_key_data) {
+    key_data <- group_data(group_by(tbl, !!! key_vars, .drop = .drop))
+  }
+  if (is_false(ordered)) { # false returns a warning
+    indices <- tbl[[index]]
+    idx_rows <- lapply(key_data[[".rows"]], function(x) indices[x])
+    actually_ordered <- all(map_lgl(idx_rows, is_ascending))
+    if (is_false(actually_ordered)) {
+      idx_txt <- backticks(index)
+      key_txt <- map(key_vars, expr_label)
+      warn(sprintf(
+        "Unspecified temporal ordering may yield unexpected results.\nSuggest to sort by %s first.",
+        comma(c(key_txt, idx_txt), sep = ""))
+      )
+    }
+    ordered <- actually_ordered
+  }
+  # validate tbl_ts
   if (validate) {
     tbl <- validate_tsibble(data = tbl, key = key_vars, index = index)
   }
   build_tsibble_meta(
-    tbl, key = key_data, index = !! index, index2 = !! index2,
-    regular = regular, ordered = ordered, interval = interval
+    tbl, key_data = key_data, index = index, index2 = index2,
+    ordered = ordered, interval = interval
   )
 }
 
 build_tsibble_meta <- function(
-  x, key, index, index2, ordered = NULL, regular = TRUE, interval = NULL
+  x, key_data = NULL, index, index2, ordered = NULL, interval = TRUE
 ) {
-  if (is_null(regular)) abort("Argument `regular` must not be `NULL`.")
-
-  index <- get_expr(enquo(index))
-  index2 <- enquo(index2)
-  if (quo_is_missing(index2)) {
-    index2 <- index
-  } else {
-    index2 <- get_expr(index2)
-  }
-  is_grped <- dplyr::is_grouped_df(x)
+  stopifnot(is_string(index) && is_string(index2))
+  stopifnot(!is_null(ordered))
   tbl <- as_tibble(x)
+  attr(index, "ordered") <- ordered
 
-  if (NROW(x) == 0) {
-    if (is_false(regular)) {
+  is_interval <- inherits(interval, "interval")
+  msg_interval <- "Argument `interval` must be class interval, not %s."
+  if (NROW(tbl) == 0) {
+    if (is_false(interval)) {
       interval <- irregular()
-    } else {
+    } else if (is_true(interval)) {
       interval <- init_interval()
+    } else if (!is_interval) {
+      abort(sprintf(msg_interval, class(interval)[1]))
     }
-    grp_data <- group_data(tbl)
-    tbl <- new_tibble(
-      tbl, "key" = key, "index" = index, "index2" = index2,
-      "interval" = interval, "regular" = regular, "ordered" = TRUE,
-      "groups" = NULL, nrow = 0L, class = "tbl_ts"
-    )
-    if (is_grped) {
-      cls <- c("grouped_ts", "grouped_df")
-      tbl <- new_tsibble(tbl, "groups" = grp_data, class = cls)
+  } else {
+    if (is_false(interval) || is_null(interval)) {
+      interval <- irregular()
+    } else if (is_true(interval)) {
+      interval <- interval_pull(tbl[[index]])
+    } else if (!is_interval) {
+      abort(sprintf(msg_interval, class(interval)[1]))
     }
-    return(tbl)
   }
 
-  if (is_false(regular)) {
-    interval <- irregular()
-  } else if (regular && is_null(interval)) {
-    eval_idx <- eval_tidy(index, data = tbl)
-    interval <- pull_interval(eval_idx)
-  } else if (is_false(inherits(interval, "interval"))) {
-    abort(sprintf(
-      "Argument `interval` must be class interval, not %s.",
-      class(interval)[1]
-    ))
-  }
-
-  idx_lgl <- identical(index, index2)
+  idx_lgl <- index == index2
   # convert grouped_df to tsibble:
   # the `groups` arg must be supplied, otherwise returns a `tbl_ts` not grouped
   if (!idx_lgl) {
-    tbl <- group_by(tbl, !! index2, add = TRUE)
+    tbl <- group_by(tbl, !! sym(index2), add = TRUE)
   }
   grp_data <- group_data(tbl)
   tbl <- new_tibble(
-    tbl, "key" = key, "index" = index, "index2" = index2,
-    "interval" = interval, "regular" = regular, "ordered" = ordered,
-    "groups" = NULL, nrow = NROW(tbl), class = "tbl_ts"
+    tbl, "key" = key_data, "index" = index, "index2" = index2,
+    "interval" = interval, "groups" = NULL, nrow = NROW(tbl), class = "tbl_ts"
   )
+  is_grped <- dplyr::is_grouped_df(x) || !idx_lgl
   if (is_grped) {
+    grp_vars <- head(names(grp_data), -1L)
+    if (index %in% grp_vars) {
+      abort(sprintf("Column `%s` (index) can't be a grouping variable.", index))
+    }
     cls <- c("grouped_ts", "grouped_df")
     tbl <- new_tsibble(tbl, "groups" = grp_data, class = cls)
   }
@@ -487,31 +341,21 @@ build_tsibble_meta <- function(
 #' @export
 new_tsibble <- function(x, ..., class = NULL) {
   not_tsibble(x)
-  x <- new_tibble(x, ..., class = "tbl_ts")
+  x <- new_tibble(x, ..., nrow = NROW(x), class = "tbl_ts")
   assert_key_data(x %@% key)
   attr(x, "row.names") <- .set_row_names(NROW(x))
   class(x) <- c(class, class(x))
   x
 }
 
-#' Identifiers used for creating key
-#'
-#' @param ... Variables passed to tsibble()/as_tsibble().
-#'
-#' @keywords internal
-#' @export
-id <- function(...) {
-  unname(enexprs(...))
-}
-
 ## Although the "index" arg is possible to automate the detection of time
 ## objects, it would fail when tsibble contain multiple time objects.
 validate_index <- function(data, index) {
   val_idx <- map_lgl(data, index_valid)
+  index <- enquo(index)
   if (quo_is_null(index)) {
     abort("Argument `index` must not be `NULL`.")
-  }
-  if (quo_is_missing(index)) {
+  } else if (quo_is_missing(index)) {
     if (sum(val_idx, na.rm = TRUE) != 1) {
       abort("Can't determine index and please specify argument `index`.")
     }
@@ -519,11 +363,11 @@ validate_index <- function(data, index) {
     chr_index <- chr_index[!is.na(chr_index)]
     inform(sprintf("Using `%s` as index variable.", chr_index))
   } else {
-    chr_index <- tidyselect::vars_pull(names(data), !! index)
+    chr_index <- vars_pull(names(data), !! index)
     idx_pos <- names(data) %in% chr_index
     val_lgl <- val_idx[idx_pos]
     if (is.na(val_lgl)) {
-      return(sym(chr_index))
+      return(chr_index)
     } else if (!val_idx[idx_pos]) {
       cls_idx <- map_chr(data, ~ class(.)[1])
       abort(sprintf("Unsupported index type: %s", cls_idx[idx_pos]))
@@ -532,22 +376,7 @@ validate_index <- function(data, index) {
   if (anyNA(data[[chr_index]])) {
     abort(sprintf("Column `%s` (index) must not contain `NA`.", chr_index))
   }
-  sym(chr_index)
-}
-
-duplicated_key_index <- function(data, key, index) {
-  # NOTE: bug in anyDuplicated.data.frame() (fixed in R 3.5.0)
-  # identifiers <- c(key, idx)
-  # below calls anyDuplicated.data.frame():
-  # time zone associated with the index will be dropped,
-  # e.g. nycflights13::weather, thus result in duplicates.
-  # dup <- anyDuplicated(data[, identifiers, drop = FALSE])
-  res <- 
-    summarise(
-      grouped_df(as_tibble(data), key),
-      !! "zzz" := anyDuplicated.default(!! index)
-    )
-  any_not_equal_to_c(res$zzz, 0)
+  chr_index
 }
 
 # check if a comb of key vars result in a unique data entry
@@ -590,7 +419,9 @@ as_tibble.tbl_ts <- function(x, ...) {
 }
 
 as_tibble.grouped_ts <- function(x, ...) {
-  as_grouped_df(x)
+  x <- remove_tsibble_attrs(x)
+  class(x) <- c("grouped_df", "tbl_df", "tbl", "data.frame")
+  as_tibble(x, ...)
 }
 
 as_tibble.grouped_df <- function(x, ...) {
@@ -618,37 +449,6 @@ as.data.frame.tbl_ts <- function(x, row.names = NULL, optional = FALSE, ...) {
   NextMethod()
 }
 
-use_id <- function(x, key) {
-  key <- enquo(key)
-  if (quo_is_call(key)) {
-    call_fn <- call_name(key)
-    if (call_fn %in% c("key", "key_data")) return(eval_tidy(key)) # key(x)
-    if (call_fn != "id") {
-      abort(sprintf("Please use `key = id(...)`, not `%s(...)`.", call_fn))
-    } # vars(x)
-  }
-  key_expr <- get_expr(key)
-  if (is_string(key_expr)) {
-    abort(suggest_key(key_expr))
-  } 
-  safe_key <- purrr::safely(eval_tidy)(
-    key_expr,
-    env = child_env(get_env(key), id = id)
-  )
-  res <- safe_key$result
-  if (is.data.frame(res)) {
-    assert_key_data(res)
-    return(res)
-  } else if (is_null(safe_key$error)) {
-    fn <- function(x) {
-      if (is_list(x)) all(map_lgl(x, fn)) else is_expression(x)
-    }
-    lgl <- fn(res)
-    if (lgl) return(res)
-  }
-  abort(suggest_key(as_string(key_expr)))
-}
-
 #' Test duplicated observations determined by key and index variables
 #'
 #' * `is_duplicated()`: a logical scalar if the data exist duplicated observations.
@@ -666,13 +466,13 @@ use_id <- function(x, key) {
 #'   fruit = c(rep(c("kiwi", "cherry"), each = 3), "cherry"),
 #'   kilo = sample(1:10, size = 7)
 #' )
-#' is_duplicated(harvest, key = id(fruit), index = year)
-#' are_duplicated(harvest, key = id(fruit), index = year)
-#' are_duplicated(harvest, key = id(fruit), index = year, from_last = TRUE)
-#' duplicates(harvest, key = id(fruit), index = year)
-is_duplicated <- function(data, key = id(), index) {
+#' is_duplicated(harvest, key = fruit, index = year)
+#' are_duplicated(harvest, key = fruit, index = year)
+#' are_duplicated(harvest, key = fruit, index = year, from_last = TRUE)
+#' duplicates(harvest, key = fruit, index = year)
+is_duplicated <- function(data, key = NULL, index) {
   key <- use_id(data, !! enquo(key))
-  index <- validate_index(data, enquo(index))
+  index <- sym(validate_index(data, !! enquo(index)))
 
   duplicated_key_index(data, key = key, index = index)
 }
@@ -682,9 +482,9 @@ is_duplicated <- function(data, key = id(), index) {
 #'
 #' @rdname duplicates
 #' @export
-are_duplicated <- function(data, key = id(), index, from_last = FALSE) {
+are_duplicated <- function(data, key = NULL, index, from_last = FALSE) {
   key <- use_id(data, !! enquo(key))
-  index <- validate_index(data, enquo(index))
+  index <- sym(validate_index(data, !! enquo(index)))
 
   res <- 
     mutate(
@@ -696,9 +496,9 @@ are_duplicated <- function(data, key = id(), index, from_last = FALSE) {
 
 #' @rdname duplicates
 #' @export
-duplicates <- function(data, key = id(), index) {
+duplicates <- function(data, key = NULL, index) {
   key <- use_id(data, !! enquo(key))
-  index <- validate_index(data, enquo(index))
+  index <- sym(validate_index(data, !! enquo(index)))
 
   ungroup(
     filter(
@@ -709,9 +509,25 @@ duplicates <- function(data, key = id(), index) {
   )
 }
 
+duplicated_key_index <- function(data, key, index) {
+  # NOTE: bug in anyDuplicated.data.frame() (fixed in R 3.5.0)
+  # identifiers <- c(key, idx)
+  # below calls anyDuplicated.data.frame():
+  # time zone associated with the index will be dropped,
+  # e.g. nycflights13::weather, thus result in duplicates.
+  # dup <- anyDuplicated(data[, identifiers, drop = FALSE])
+  res <- 
+    summarise(
+      grouped_df(as_tibble(data), key),
+      !! "zzz" := anyDuplicated.default(!! sym(index))
+    )
+  any_not_equal_to_c(res$zzz, 0)
+}
+
 remove_tsibble_attrs <- function(x) {
   attr(x, "key") <- attr(x, "index") <- attr(x, "index2") <- NULL
-  attr(x, "interval") <- attr(x, "regular") <- attr(x, "ordered") <- NULL
+  # attr(x, "interval") <- NULL
+  attr(x, "interval") <- attr(x, "ordered") <- attr(x, "regular") <- NULL # will be removed
   x
 }
 
