@@ -17,8 +17,7 @@ test_that("dplyr verbs for empty tsibble", {
   expect_equal(NROW(ped_null %>% mutate(Count1 = Count + 1)), 0L)
   expect_equal(NROW(ped_null %>% transmute(Count1 = Count + 1)), 0L)
   expect_equal(NROW(ped_null %>% summarise(Count1 = sum(Count))), 0L)
-  expect_warning(ped_null %>% arrange(Count), "Unexpected temporal ordering.")
-  # expect_identical(ped_null %>% arrange(Count), ped_null)
+  expect_identical(ped_null %>% arrange(Count), ped_null)
   expect_identical(ped_null %>% slice(0), ped_null)
   expect_identical(ped_null %>% filter(Sensor == 0), ped_null)
   expect_identical(ped_null[0, ], ped_null)
@@ -77,3 +76,19 @@ test_that("dplyr verbs for empty groups (characters)", {
     group_data(ped_grped %>% arrange(Sensor, Date_Time))
   )
 })
+
+x <- tibble(
+  groups = rep(factor("a", levels = letters[1:2]), 10),
+  index = as.Date("2019-01-01") + 0:9
+)
+
+test_that("dplyr verbs for empty groups (factors)", {
+  y <- as_tsibble(x, key = groups, .drop = FALSE)
+  expect_equal(n_keys(y), 2L)
+  expect_false(key_drop_default(y))
+  expect_equal(n_keys(y %>% mutate(a = 1)), 2L)
+  expect_equal(n_keys(y %>% mutate(groups = groups)), 2L)
+  expect_equal(n_keys(y %>% mutate(groups = 1L)), 1L)
+  expect_equal(n_keys(y %>% select(index, groups)), 2L)
+  expect_equal(n_keys(y %>% rename(index2 = index, groups2 = groups)), 2L)
+})
