@@ -26,8 +26,11 @@ test_that("units_since()", {
 test_that("diff()", {
   expect_is(diff(yw), "difftime")
   expect_equal(as.numeric(diff(yw)), rep(1, 2))
+  expect_identical(interval_pull(diff(yw) + 0:1), new_interval(week = 1L))
   expect_equal(as.numeric(diff(ym)), rep(1, 2))
+  expect_identical(interval_pull(diff(ym) + c(2, 4)), new_interval(month = 2L))
   expect_equal(as.numeric(diff(yq)), rep(1, 2))
+  expect_identical(interval_pull(diff(yq) + c(3, 6)), new_interval(quarter = 3L))
 })
 
 test_that("Arit", {
@@ -36,19 +39,25 @@ test_that("Arit", {
   expect_equal(format(1 + yw), format(yw + 1))
   expect_equal(format(yw - 1), c("1969 W52", "1970 W01", "1970 W02"))
   expect_error(yw + yw, "not defined")
-  expect_error(yw - yw, "not defined")
+  expect_equal(as.integer(yw - yw), rep(0, 3))
+  expect_equal(as.integer(yw[2] - yw[1]), 1)
+  expect_equal(as.integer(yw[1] - yw[2]), -1)
   expect_equal(format(+ ym), format(ym))
   expect_equal(format(ym + 1), c("1970 Feb", "1970 Mar", "1970 Apr"))
   expect_equal(format(1 + ym), format(ym + 1))
   expect_equal(format(ym - 1), c("1969 Dec", "1970 Jan", "1970 Feb"))
   expect_error(ym + ym, "not defined")
-  expect_error(ym - ym, "not defined")
+  expect_equal(as.integer(ym - ym), rep(0, 3))
+  expect_equal(as.integer(ym[2] - ym[1]), 1)
+  expect_equal(as.integer(ym[1] - ym[2]), -1)
   expect_equal(format(+ yq), format(yq))
   expect_equal(format(yq + 1), c("1970 Q2", "1970 Q3", "1970 Q4"))
   expect_equal(format(1 + yq), format(yq + 1))
   expect_equal(format(yq - 1), c("1969 Q4", "1970 Q1", "1970 Q2"))
   expect_error(yq + yq, "not defined")
-  expect_error(yq - yq, "not defined")
+  expect_equal(as.integer(yq - yq), rep(0, 3))
+  expect_equal(as.integer(yq[2] - yq[1]), 1)
+  expect_equal(as.integer(yq[1] - yq[2]), -1)
 })
 
 a <- yearweek(seq(ymd("2017-02-01"), length.out = 12, by = "1 week"))
@@ -96,3 +105,24 @@ test_that("character", {
   expect_equal(format(yearmonth("201801")), "2018 Jan")
   expect_equal(format(yearquarter(as.character(xx))), "2018 Q1")
 })
+
+test_that("yearmonth() #89", {
+  expect_false(
+    anyNA(yearmonth(as.numeric(time(
+      ts(rnorm(139), frequency = 12, start = c(1978, 2))
+    ))))
+  )
+})
+
+test_that("yearquarter() for characters #107", {
+  expect_error(yearquarter("2013 Qt 3"), "cannot be expressed as Date type")
+  expect_error(yearquarter("Qtrr 5 2015"), "cannot be expressed as Date type")
+  expect_error(yearquarter("Quar 5 2015"), "cannot be expressed as Date type")
+  expect_error(yearquarter("Q5 2015"), "can't be greater than 4.")
+  expect_error(yearquarter("Q2015"), "unambiguous")
+  expect_error(yearquarter(c("Q2015", "Q2 2015")), "unambiguous")
+  expect_identical(
+    yearquarter(c("2013 Q3", "2013 Qtr 3", "Quarter 3 2013")),
+    rep(yearquarter("2013 Q3"), 3)
+  )
+})
