@@ -4,7 +4,7 @@ harvest <- tsibble(
   year = rep(2010:2012, 2),
   fruit = rep(c("kiwi", "cherry"), each = 3),
   kilo = sample(1:10, size = 6),
-  key = id(fruit), index = year
+  key = fruit, index = year
 )
 
 test_that("error for existing `.id`", {
